#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : completions
# @Time         : 2023/12/19 16:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 逆向工程

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from sse_starlette import EventSourceResponse
from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status
from fastapi.responses import RedirectResponse

from openai import OpenAI, AsyncOpenAI
from openai.types.chat import ChatCompletion, ChatCompletionChunk

from chatllm.utils.openai_utils import to_openai_completion_params, openai_response2sse
from chatllm.llmchain.completions import moonshot_kimi
from chatllm.llmchain.completions import deepseek
from chatllm.llmchain.applications import chaturl

from chatllm.schemas.openai_types import chat_completion_per, chat_completion_slogan, chat_completion_chunk_slogan
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo

router = APIRouter()

ChatCompletionResponse = Union[ChatCompletion, List[ChatCompletionChunk]]

send_message = lru_cache(send_message)


@router.post("/chat/completions")
async def create_chat_completions(
    request: ChatCompletionRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    """广告、按次计费、兜底"""
    api_key = auth and auth.credentials or None
    if api_key is None:
        detail = {
            "error": {
                "message": "",
                "type": "invalid_request_error",
                "param": None,
                "code": "invalid_api_key",
            }
        }
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail=detail)

    # 空服务 按次计费 per
    if request.model.startswith(("per", "file-extract", "ocr")): return chat_completion_per

    # todo
    # if request.model.startswith(('url-',)):
    #     request.model = request.model.strip('url-')
    #     response = chaturl.Chat(api_key=api_key).create(request)  # todo: 优化
    #     return completions

    data = to_openai_completion_params(request)
    response = OpenAI(
        api_key=api_key,
        base_url='https://api.chatllm.vip/v1',
        http_client=httpx.Client(follow_redirects=True)
    ).chat.completions.create(**data)

    if request.stream:
        _ = itertools.chain((chat_completion_chunk_slogan,), response)
        return openai_response2sse(_)

    return response


@router.post("/redirect/v1/chat/completions")
async def create_chat_completions_for_redirect(
    request: ChatCompletionRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    """
    1、重定向可以避免跨域问题
    """
    return RedirectResponse(url="/chat/completions")


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/v1')

    app.run()
    # for i in range(10):
    #     send_message(f"兜底模型", title="github_copilot")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : st_chat
# @Time         : 2023/8/11 14:45
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

import streamlit as st
# st.set_page_config('🔥ChatLLM', layout='wide', initial_sidebar_state='collapsed')

from chatllm.llmchain import init_cache
from chatllm.llmchain.applications import ChatFiles

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.serving.streamlit import hide_st_style, st_chat_message, ChatMessage

send_message = partial(
    send_message,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/f0475882-ad39-49d9-ad77-523c0e768e96"
)

hide_st_style()

init_cache(1)


@st.cache_resource(show_spinner=False)
def fn(file_path=None, file=None):
    chatfile = (
        ChatFiles(
            model="gpt-3.5-turbo",
            embedding_model="text-embedding-ada-002",
        )
        .load_file(file_path=file_path, file=file, chunk_size=500)
    )

    return chatfile


# chatfile = fn("附件1：东北证券股份有限公司员工合规手册（2023年12月）.docx")

if __name__ == '__main__':
    col1, col2, col3, *_ = st.columns(3)
    with col1:
        st.image('规丞相.png', width=64)
    with col2:
        st.markdown('##### ')
        st.markdown('##### ')
        st.markdown('##### 规丞相驾到')

    st.markdown('> ⚠️“规丞相”仅供东北证券内部测试使用，所做回答不得用于东北证券官方回复。')
    file_placeholder = st.empty()

    st_chat_message(ChatMessage(avatar="规丞相.png", generator="欢迎来找**规丞相**，您有什么要咨询的吗❓"))

    for message in st.session_state.messages:  # 设计his
        st_chat_message(message)

    print(st.session_state.messages)

    file = file_placeholder.file_uploader("文档问答")
    if file:
        with open(file.name, 'wb') as out:
            out.write(file.read())
        chatfile = fn(file.name)
    else:
        chatfile = fn("附件1：东北证券股份有限公司员工合规手册（2023年12月）.docx")

    prompt = st.chat_input("    🤔 你可以问我任何问题", key='xx')  # 最下面
    if prompt:
        send_message(prompt)
        with st.spinner('AI 🤔'):
            st_chat_message(ChatMessage(name="user", generator=prompt), is_history=False)

            # st_chat_message(
            #     ChatMessage(
            #         avatar="!.png",
            #         generator="`大模型根据公司相关法规及提问语义做答，不作为合规管理官方回复。`"
            #     )
            # )

            generator = chatfile.create(prompt, debug=True)
            nesc = "<font color='blue'>【大模型根据公司相关法规及提问语义做答，不作为合规管理官方回复。】</font>"


            def gen():
                yield nesc
                yield from generator


            st_chat_message(ChatMessage(avatar="规丞相.png", generator=gen()), is_history=False, unsafe_allow_html=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ppu_utils
# @Time         : 2024/1/8 16:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 按次计费

from openai import OpenAI

from meutils.pipe import *
from chatllm.schemas.openai_types import completion_keys
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest

from openai.types.chat import ChatCompletion, ChatCompletionChunk


def to_openai_completion_params(
    request: Union[dict, ChatCompletionRequest],
    redirect_model: Optional[str] = None
) -> dict:
    data = copy.deepcopy(request)
    if isinstance(request, ChatCompletionRequest):
        data = request.model_dump()

    extra_body = {}
    for key in list(data):
        if key not in completion_keys:
            extra_body.setdefault(key, data.pop(key))

    data['extra_body'] = extra_body  # 拓展字段
    data['model'] = redirect_model or data['model']

    return data


# @background_task
def per_create(api_key: Optional[str] = None, base_url: Optional[str] = None, model: str = 'per'):
    openai = OpenAI(
        api_key=api_key,
        base_url=base_url or "https://api.chatllm.vip/v1",
        http_client=httpx.Client(follow_redirects=True)
    )
    return openai.chat.completions.create(model=model, messages=[{"role": "user", "content": "hi"}])


def openai_response2sse(response, redirect_model: Optional[str] = None, n: float = 1.2):
    """

    :param response:
    :param redirect_model: request.model
    :param n:
    :return:
    """
    from sse_starlette import EventSourceResponse

    # logger.debug(response)

    if isinstance(response, ChatCompletion):
        response.model = redirect_model or response.model
        response.usage.total_tokens = int(response.usage.total_tokens * n)
        response.usage.prompt_tokens = int(response.usage.prompt_tokens * n)
        response.usage.completion_tokens = int(response.usage.completion_tokens * n)
        return response

    def gen():
        for chunk in response:
            if not (chunk.choices and chunk.choices[0].delta.content):  # 都跑错的时候如何兜底
                continue

            # print(chunk)

            chunk: ChatCompletionChunk
            chunk.model = redirect_model or chunk.model
            if chunk.choices[0].finish_reason == "stop":
                chunk.choices[0].delta.content = ""  # "delta":{} "delta":{"content":""}
                yield chunk.model_dump_json()
                yield "[DONE]"  # 兼容标准格式
                return
            else:
                yield chunk.model_dump_json()

    return EventSourceResponse(gen(), ping=10000)  # sse_response


if __name__ == '__main__':
    print(per_create())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : summarize
# @Time         : 2023/8/9 14:44
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from langchain.text_splitter import *
from langchain.schema.language_model import BaseLanguageModel
from langchain.prompts import ChatPromptTemplate
from langchain_community.chat_models import ChatOpenAI
from langchain.chains.summarize import load_summarize_chain

# ME
from meutils.pipe import *
from chatllm.llmchain.decorators import llm_stream
from chatllm.llmchain.document_loaders import FilesLoader
from chatllm.llmchain.prompts.prompt_templates import summary_prompt_template, question_generation_prompt_template


class Summarizer(object):

    def __init__(self, llm: Optional[BaseLanguageModel] = None):
        self.llm = llm or ChatOpenAI(model="gpt-3.5-turbo-16k-0613", temperature=0, streaming=True)

    @logger.catch
    def generate(self, docs: List[Document], max_tokens: int = 10000, prompt_template: str = summary_prompt_template):
        chain_type = 'stuff'
        if sum((len(doc.page_content) for doc in docs)) > max_tokens:
            chain_type = 'map_reduce'

        logger.debug(chain_type)

        self.chain = load_summarize_chain(
            self.llm,
            chain_type=chain_type,
            prompt=ChatPromptTemplate.from_template(prompt_template)
        )
        return self.chain.run(docs)

    @staticmethod
    def load_file(
        file_paths,
        max_workers=3,
        chunk_size=2000,
        chunk_overlap=200,
        separators: Optional[List[str]] = None
    ) -> List[Document]:
        """支持多文件"""
        loader = FilesLoader(file_paths, max_workers=max_workers)
        separators = separators or ['\n\n', '\r', '\n', '\r\n', '。', '!', '！', '\\?', '？', '……', '…']
        textsplitter = RecursiveCharacterTextSplitter(
            chunk_size=chunk_size,
            chunk_overlap=chunk_overlap,
            add_start_index=True,
            separators=separators
        )
        docs = loader.load_and_split(textsplitter)
        return docs


if __name__ == '__main__':
    s = Summarizer()

    docs = s.load_file('/Users/betterme/PycharmProjects/AI/ChatLLM/data/姚明.txt')

    # print(s.generate(docs))
    # print(s.generate(docs, prompt_template=question_generation_prompt_template))

    chain = load_summarize_chain(
        s.llm,
        prompt=ChatPromptTemplate.from_template(summary_prompt_template)
    )
    # print(chain.run(docs))
    # print(chain.run({'input_documents': docs, "question": None}))
    print(chain.run({'input_documents': docs, "question": '你是谁？'}))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : st_chat
# @Time         : 2023/8/11 14:45
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

import streamlit as st
# st.set_page_config('🔥ChatLLM', layout='wide', initial_sidebar_state='collapsed')

from chatllm.llmchain import init_cache
from chatllm.llmchain.applications import ChatFiles

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.serving.streamlit import hide_st_style, st_chat_message, ChatMessage

send_message = partial(
    send_message,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/d9c14c9b-7572-42a5-9318-463bc10768a0"
)

hide_st_style()

init_cache(1)

st.sidebar.markdown("##### [必填](https://api.chatfire.cn/)")

openai_api_key = st.sidebar.text_input("api_key", placeholder="sk-...").strip()
openai_api_base = st.sidebar.text_input("base_url", placeholder="https://api.chatfire.cn/v1")

if openai_api_key and len(openai_api_key) != 51:
    st.sidebar.error("请填写正确的api_key")


@st.cache_resource(show_spinner=False)
def fn(file_path=None, file=None):
    chatfile = (
        ChatFiles(
            model="gpt-3.5-turbo",
            embedding_model="text-embedding-ada-002",
            openai_api_key=openai_api_key,
            openai_api_base=openai_api_base
        )
        .load_file(file_path=file_path, file=file, chunk_size=500)
    )

    return chatfile


if __name__ == '__main__':
    col1, col2, col3, *_ = st.columns(3)

    st.markdown('##### 🔥Chatfire RAG')
    file_placeholder = st.empty()

    st_chat_message(ChatMessage(avatar='fire.png', generator="我是文件问答助手，请问有什么可以帮助你❓"))

    for message in st.session_state.messages:  # 设计his
        st_chat_message(message)

    file = file_placeholder.file_uploader("📃请上传文件")
    chatfile = None
    if file:
        with open(file.name, 'wb') as out:
            out.write(file.read())
        chatfile = fn(file.name)

    prompt = st.chat_input("    🤔 你可以问我任何问题", key='xx')  # 最下面
    if prompt:
        send_message(prompt)
        with st.spinner('AI 🤔'):
            st_chat_message(ChatMessage(name="user", generator=prompt), is_history=False)

            if chatfile:
                generator = chatfile.create(prompt, debug=True)

                st_chat_message(ChatMessage(avatar='fire.png', generator=generator), is_history=False,
                                unsafe_allow_html=True)

            else:

                st_chat_message(ChatMessage(avatar='fire.png', generator="**请先上传文件**"), is_history=False,
                                unsafe_allow_html=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : app
# @Time         : 2023/11/13 17:34
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 模型转发服务

from meutils.pipe import *

from fastapi import APIRouter, File, UploadFile, Query, Form
from sse_starlette import EventSourceResponse
from langchain_community.chat_models import ChatOpenAI
from langchain_community.chat_models.openai import acompletion_with_retry  # llm.completion_with_retry
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.schemas.openai_api_protocol import ChatCompletionResponse, ChatCompletionStreamResponse

router = APIRouter()


# @app.post("/chat/completions", dependencies=[Depends(check_api_key)])
@router.post("/v1/chat/completions")
async def create_chat_completions(request: ChatCompletionRequest):
    """Creates a completion for the chat message"""

    if isinstance(request.messages, str):
        messages = [{'role': 'user', 'content': request.messages}]
    else:
        messages = request.messages

    return ChatCompletionResponse(**r)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : kb
# @Time         : 2023/9/25 17:51
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

"""
You are an expert document question answering system. You answer questions by finding relevant content in the document and answering questions based on that content.
Document: <textual metadata of document>


您是文档问答系统专家。您可以通过在文档中查找相关内容并根据该内容回答问题来回答问题。文档:<文档的文本元数据>
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : docx
# @Time         : 2023/8/15 17:08
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from langchain.text_splitter import *
from langchain.schema import Document
from langchain_community.document_loaders import Docx2txtLoader as _Docx2txtLoader

from meutils.pipe import *
from meutils.fileparser import stream_parser


class Docx2txtLoader(_Docx2txtLoader):
    def __init__(self, file_path: Any) -> None:
        """Initialize with a file path."""
        try:
            import docx2txt  # noqa:F401
        except ImportError:
            raise ImportError(
                "`PyMuPDF` package not found, please install it with "
                "`pip install docx2txt`"
            )

        self.file_path = file_path

    def load(self) -> List[Document]:

        if (
            isinstance(self.file_path, (str, os.PathLike))
            and len(self.file_path) < 256
            and Path(self.file_path).is_file()
        ):
            return _Docx2txtLoader(self.file_path).load()

        import docx2txt

        filename, file_stream = stream_parser(self.file_path)
        return [
            Document(
                page_content=docx2txt.process(io.BytesIO(file_stream)),
                metadata={"source": filename},
            )
        ]


if __name__ == '__main__':
    p = '/Users/betterme/PycharmProjects/AI/ChatLLM/data/吉林碳谷报价材料.docx'
    print(Docx2txtLoader(p).load())
    print(Docx2txtLoader(open(p)).load())
    print(Docx2txtLoader(open(p, 'rb')).load())
    print(Docx2txtLoader(open(p, 'rb').read()).load())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : vdb
# @Time         : 2023/8/7 13:42
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from langchain.docstore.document import Document
from langchain_community.vectorstores import DocArrayInMemorySearch as _DocArrayInMemorySearch


class DocArrayInMemorySearch(_DocArrayInMemorySearch):

    def similarity_search(
        self,
        query: str,
        k: int = 4,
        threshold: float = 0.5,
        **kwargs: Any,
    ) -> List[Document]:

        docs_scores = self.similarity_search_with_score(query=query, k=k, **kwargs)

        docs = []
        for doc, score in docs_scores:
            if score > threshold:
                doc.metadata['score'] = round(score, 2)
                docs.append(doc)
        return docs



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : DashScopeEmbeddings
# @Time         : 2023/7/27 13:37
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

import langchain
from langchain_community.embeddings.dashscope import DashScopeEmbeddings as _DashScopeEmbeddings

from meutils.pipe import *
from meutils.np_utils import normalize
from chatllm.llmchain.utils import get_api_key


class DashScopeEmbeddings(_DashScopeEmbeddings):
    chunk_size: int = 25
    """Maximum number of texts to embed in each batch"""
    show_progress_bar: bool = False
    """Whether to show a progress bar when embedding."""
    normalize_embeddings: bool = True
    """多key多线程"""
    get_api_key: Callable[[int], List[str]] = get_api_key

    def embed_documents(
        self, texts: List[str], chunk_size: Optional[int] = 0
    ) -> List[List[float]]:
        n = int(np.ceil(len(texts) / self.chunk_size))
        api_key_set = self.get_api_key(n=n)

        max_workers = np.clip(len(api_key_set), 1, 16).astype(int)  # 最大线程数
        if max_workers > 1:
            embeddings_map = {}
            for i, api_key in enumerate(api_key_set):
                kwargs = self.dict().copy()
                kwargs['dashscope_api_key'] = api_key
                embeddings_map[i] = DashScopeEmbeddings(**kwargs)  # 多个对象实例

            if langchain.debug:
                logger.info([e.dashscope_api_key for e in embeddings_map.values()])
                logger.info(f"Maximum concurrency: {max_workers * self.chunk_size}")

            def __embed_documents(arg):
                idx, texts = arg
                embeddings = embeddings_map.get(idx % max_workers, 0)
                return embeddings._embed_documents(texts)

            return (
                texts | xgroup(self.chunk_size)
                | xenumerate
                | xThreadPoolExecutor(__embed_documents, max_workers)
                | xchain_
            )

        return self._embed_documents(texts)

    def _embed_documents(self, texts: List[str], chunk_size=None) -> List[List[float]]:
        """Call out to DashScope's embedding endpoint for embedding search docs.

        Args:
            texts: The list of texts to embed.
            chunk_size: The chunk size of embeddings. If None, will use the chunk size
                specified by the class.

        Returns:
            List of embeddings, one for each text.
        """

        batched_embeddings = []
        _chunk_size = chunk_size or self.chunk_size

        if self.show_progress_bar:
            _iter = tqdm(range(0, len(texts), _chunk_size))
        else:
            _iter = range(0, len(texts), _chunk_size)

        for i in _iter:
            response = embed_with_retry(
                self,
                input=texts[i: i + _chunk_size],
                text_type="document",
                model=self.model,
                # api_key=api_key
            )
            batched_embeddings += [r["embedding"] for r in response]  # response: embeddings

        return batched_embeddings if not self.normalize_embeddings else normalize(np.array(batched_embeddings)).tolist()

    def embed_query(self, text: str) -> List[float]:
        embedding = super().embed_query(text)
        return embedding if not self.normalize_embeddings else normalize(np.array(embedding)).tolist()

if __name__ == '__main__':
    print(DashScopeEmbeddings().embed_query(text='a'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatdoc
# @Time         : 2023/7/15 20:53
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo:多轮对话

from chatllm.llmchain.decorators import llm_stream
from chatllm.llmchain.vectorstores import FAISS
from chatllm.llmchain.embeddings import OpenAIEmbeddings
from chatllm.llmchain.prompts import rag
from chatllm.llmchain.utils import tiktoken_encoder
from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk

from langchain.text_splitter import *
from langchain_community.chat_models import ChatOpenAI
from langchain.chains.question_answering import load_qa_chain
from langchain.prompts import ChatPromptTemplate
from langchain_community.document_loaders import UnstructuredFileLoader
from chatllm.llmchain.document_loaders import FileLoader

from meutils.pipe import *
from meutils.notice.feishu import send_message
from typing import IO


class ChatFiles(object):

    def __init__(
        self,
        model="gpt-3.5-turbo",
        # embedding_model="text-embedding-ada-002",
        embedding_model="jina-embedding",
        openai_api_key: Optional[str] = None,
        openai_api_base: Optional[str] = None,
        stream: bool = True,
        temperature: float = 0,
        use_ann: Optional[bool] = None,
        max_tokens: Optional[int] = 10000,
        prompt_template: Optional[ChatPromptTemplate] = None,
        **kwargs
    ):
        openai_api_key = openai_api_key or os.getenv("OPENAI_API_KEY")
        openai_api_base = openai_api_base or os.getenv("OPENAI_API_BASE", "https://api.chatllm.vip/v1")

        self.llm = ChatOpenAI(
            model=model,
            temperature=temperature,
            streaming=stream,
            openai_api_key=openai_api_key,
            openai_api_base=openai_api_base
        )
        self.embeddings = OpenAIEmbeddings(
            chunk_size=5,  # 小一点
            model=embedding_model,
            openai_api_key=openai_api_key,
            openai_api_base=openai_api_base
        )
        self.prompt_template = prompt_template or rag.template

        self.stream = stream

        self.docs = None
        self.use_ann = use_ann
        self.max_tokens = max_tokens
        self.vectorstore = None

        self.chain = load_qa_chain(self.llm, prompt=prompt_template)  # todo: 增加信源

    def create(self, query: str, top_k: int = 5, threshold: float = 0.666, debug=False, **kwargs: Any):
        if self.vectorstore:
            self.docs = self.vectorstore.similarity_search(query, k=top_k, threshold=threshold, **kwargs)  # 优化加速

            if debug:
                # logger.debug(query)
                # self.docs | xmap_(logger.debug)
                send_message(title=query)
                self.docs | xmap(lambda doc: doc.page_content) | xmap_(send_message)

            if not self.docs:
                logger.warning(f"DocsNull: {query} -> {self.docs}")

        if self.stream:
            return llm_stream(self.chain.run)({"input_documents": self.docs, "question": query})
        else:
            return self.chain.run({"input_documents": self.docs, "question": query})

    def create_sse(self, query):
        response = self.create(query)
        if self.stream:
            def generator():
                for chunk in response:
                    chat_completion_chunk.choices[0].delta.content = chunk
                    yield chat_completion_chunk.model_dump_json()

            from sse_starlette import EventSourceResponse
            return EventSourceResponse(generator(), ping=10000)

        chat_completion.choices[0].message.content = response
        return chat_completion

    def load_file(
        self,
        file_path: Optional[str] = None,
        file: Optional[IO[bytes]] = None,
        chunk_size=1000,
        chunk_overlap=50,
        separators: Optional[List[str]] = None,
        **kwargs
    ):

        """todo: 支持多文件"""
        self.docs = UnstructuredFileLoader(
            file_path=file_path,
            file=file,
            metadata_filename=file_path and Path(file_path).name or (hasattr(file, 'name') and file.name),
            strategy="fast",
        ).load()

        if self.use_ann or len(tiktoken_encoder.encode(self.docs[0].page_content)) > self.max_tokens:
            separators = separators or ['\n\n', '\r', '\n', '\r\n', '。', '!', '！', '\\?', '？', '……', '…']
            textsplitter = RecursiveCharacterTextSplitter(
                chunk_size=chunk_size,
                chunk_overlap=chunk_overlap,
                add_start_index=True,
                separators=separators
            )
            self.docs = textsplitter.split_documents(self.docs)
            self.vectorstore = FAISS.from_documents(self.docs, self.embeddings)  # ann 索引
            return self

        logger.debug('不走ANN')

        return self


if __name__ == '__main__':
    from chatllm.llmchain import init_cache

    # init_cache()
    model: str = "text-embedding-ada-002"
    model = "jina-embedding"

    q = '已出票成功的机票，发现姓名/证件号码写错了，要如何操作？'
    # q = 'Q:为什么有的订单要收取消费？ A:不同服务商针对乘客取消订单行为可能会收取相应取消费，为避免造成额外损失，请合理安排出行计划合理下单。如遇特殊情况，请及时联系 同程商旅客服协助处理。'
    _ = (
        ChatFiles(use_ann=False, embedding_model=model)
        .load_file(
            file=open(
                '/Users/betterme/PycharmProjects/AI/ChatWecom/chatwecom/data/同程商旅Q&A通用版（机、酒、火、用车）1.1 (1)(1).pdf',
                'rb'),
            chunk_size=300,
        )
        .create(q)
    )

    for i in _:
        print(i, end='')
